from selenium import webdriver
from webdriver_manager.chrome import ChromeDriverManager
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import WebDriverWait
from selenium.common.exceptions import TimeoutException
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.by import By

import datetime
import numpy as np
import pandas as pd


class ElectricityPrice():


    def __init__(self):

        """Clase para la extracción de los precios proporcionado por Red Eléctrica de España
        por tramos horarios.
        """

        self.co = webdriver.ChromeOptions()
        self.co.add_argument("--headless")
        self.driver = webdriver.Chrome(chrome_options=self.co,executable_path=ChromeDriverManager().install())

    def extract_data(self, index_to_datetime = True):
        """Método para la extracción del DataFrame

        Arg:
            index_to_datetime (bool): Poner a True si queremos el índice en formateo DateTime.

        Returns:
            pd.DataFrame: Archivo con precios de electricidad por tramos horarios.
        """
        
        url  = 'https://tarifaluzhora.es/'
        self.driver.get(url)

        times = self.driver.find_elements(By.XPATH, "//div//span[@itemprop='description']") 
        prices = self.driver.find_elements(By.XPATH, "//div//span[@itemprop='price']")

        if index_to_datetime:
            times = list(map(lambda x: datetime.time(int(x.text.split('h')[0])), times))
            date = pd.to_datetime(datetime.datetime.today().strftime("%d/%m/%Y"))
            time = list(map(lambda x: datetime.datetime.combine(date, x) ,times))
        else:
            time = list(map(lambda x: x.text, times))
                
        price = list(map(lambda x: float(x.text.split(' ')[0]), prices))
        unit = prices[0].text.split(' ')[-1]

        df = pd.DataFrame({f'{unit}': pd.Series(price, dtype=float), 'Hour': pd.Series(time, dtype=str)}).set_index('Hour', drop=True)

        return df






from selenium import webdriver
from webdriver_manager.chrome import ChromeDriverManager
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import WebDriverWait
from selenium.common.exceptions import TimeoutException
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.by import By
import time
import os

class PVGIS():
    def __init__(self, lat, lon, azimuth, elevation, absolute_path):

        """Clase para hace scrapping de la página web de PVGIS 

        Args:
            lat (float): Latitud del lugar
            lon (float): Longitud del lugar
            azimuth (float): Azimuth de -180º a 180º
            elevation (float): Elevación de 0º a 90º
            absolute_path (string): Dirección absoluta de donde guardar el archivo
            
        """

        self.lat = lat
        self.lon = lon
        self.azimuth = azimuth
        self.elevation = elevation

        self.prefs = {'download.default_directory' : absolute_path}
        self.co = webdriver.ChromeOptions()
        self.co.add_experimental_option('prefs', self.prefs)
        self.driver = webdriver.Chrome(chrome_options=self.co,executable_path=ChromeDriverManager().install())
    
    def interact_with_page(self):
        """Función para interactuar con la página web de PVGIS
        """

        #Vamos a la página para los tramos horarios.
        url  = f'https://re.jrc.ec.europa.eu/pvg_tools/en/#HR'
        self.driver.get(url)
        
        #Seleccionamos latitud y longitud.
        latitude =  WebDriverWait(self.driver, 10).until(EC.visibility_of_element_located((By.ID, "inputLat")))
        longitude =  WebDriverWait(self.driver, 10).until(EC.visibility_of_element_located((By.ID, "inputLon")))

        latitude.send_keys(f'{self.lat}')
        longitude.send_keys(f'{self.lon}')

        go_buttons = self.driver.find_elements(By.ID, "tr_go")
        go_buttons[-1].click()
        time.sleep(2)

        #Seleccionamos azimuth y elevación.
        slope =  WebDriverWait(self.driver, 10).until(EC.visibility_of_element_located((By.ID, "hourlyangle")))
        azimuth =  WebDriverWait(self.driver, 10).until(EC.visibility_of_element_located((By.ID, "hourlyaspect")))

        slope.send_keys(f'{self.elevation}')
        azimuth.send_keys(f'{self.azimuth}')
        # time.sleep(2)

        #Seleccionamos componentes de radiación.
        WebDriverWait(self.driver, 10).until(EC.element_to_be_clickable((By.XPATH, "//input[@name='components']"))).click()
        # time.sleep(2)

        #Descargamos los datos seleecionados.
        WebDriverWait(self.driver, 10).until(EC.element_to_be_clickable((By.XPATH, "//div[@id='hourlydownloadcsv']"))).click()



from pvlib import location, irradiance
import pandas as pd


def get_irradiance(lat, lon, start_date, end_date, tilt, surface_azimuth, freq='1H'):
    # Create location object to store lat, lon, timezone
    site_location = location.Location(lat, lon)
    # Creates one day's worth of 1 hour intervals
    times = pd.date_range(start=start_date, end=end_date, freq=freq)
    # Generate clearsky data using the Ineichen model, which is the default
    # The get_clearsky method returns a dataframe with values for GHI, DNI,
    # and DHI
    clearsky = site_location.get_clearsky(times)
    # Get solar azimuth and zenith to pass to the transposition function
    solar_position = site_location.get_solarposition(times=times)
    # Use the get_total_irradiance function to transpose the GHI to POA
    POA_irradiance = irradiance.get_total_irradiance(
        surface_tilt=tilt,
        surface_azimuth=surface_azimuth,
        dni=clearsky['dni'],
        ghi=clearsky['ghi'],
        dhi=clearsky['dhi'],
        solar_zenith=solar_position['apparent_zenith'],
        solar_azimuth=solar_position['azimuth'])
    # Return DataFrame with only GHI and POA
    return pd.DataFrame({'GHI': clearsky['ghi'],
                         'POA': POA_irradiance['poa_global']})



from math import ceil
from pv_sizing.dimension.pv import PVProduction

class BatterySizing(PVProduction):

    def __init__(self, load, irr_data, fresnel_eff, tnoct, gamma, panel_power, num_panel,
                 inversor_eff, batt_volt, days_auto, dod, amp_hour_rating,
                 amb_temp_multiplier, **kwargs):
        """
        Args:
            load: pd.DataFrame
                Datos horarios de carga obtenido de https://www.edistribucion.com/
            irr_data: pd.DataFrame
                Irradiancias (directa, difusa y reflejada) sobre el plano inclinado, temperatura
                media, velcodidad del viento horarios obtenido a partir de https://re.jrc.ec.europa.eu/pvg_tools/en/
            tnoct: int
                "Nominal operating cell temperature" facilitada por el fabricante
            gamma: float
                Coeficiente de pérdidas facilitado por el fabricante
            panel_power: int
                Potencia pico del panel fotovoltaico
            num_panel: int
                Número de paneles fotovoltaicos
            fresnel_eff: array_like
                Eficiencia fresnel por cada mes
            inversor_eff: float
                Eficiencia del inversor obtenida de la hoja técnica.
            batt_volt: int
                Voltaje del banco de baterías.
            amb_temp_multiplier: float
                Multiplicador de temperatura dependiente de la temperatura mínima en invierno.
            days_auto: float
                Días de autonomía.
            dod: float
                dod de la batería obtenida de la hoja ténica.
            amp_hour_rating: float
                Amp-hour rating obtenido de la hoja ténica.
            batt_volt: int
                Voltaje [V] de una batería.
        """
        super().__init__(load=load, irr_data=irr_data, fresnel_eff=fresnel_eff, tnoct=tnoct, gamma=gamma,
                         panel_power=panel_power, num_panel=num_panel,
                         lat=None, lon=None, start_date=None, end_date=None, tilt=None,
                         surface_azimuth=None, freq='1H')

        self.inversor_eff = inversor_eff
        self.batt_volt = batt_volt
        self.days_auto = days_auto
        self.dod = dod
        self.amp_hour_rating = amp_hour_rating
        self.nominal_voltage = self.set_nominal_voltage()
        self.amb_temp_multiplier = amb_temp_multiplier

    def daily_energy(self):
        return self.mean_hourly_load_data().AE_kWh.sum()

    def daily_ah(self):
        """
        Returns: float
            Amperios-hora diarios [Ah]

        """
        return (self.daily_energy() * 1000 / self.inversor_eff) / self.batt_volt # to Ah wiht x1000

    def set_nominal_voltage(self):

        daily_energy = self.daily_energy()

        if daily_energy <= 1:
            nom_vol = 12
        elif (daily_energy > 1) and (daily_energy <= 3.5):
            nom_volt = 24
        else:
            nom_volt = 48
        
        return nom_volt

    def battery_sizing(self):
        """

        Returns: tuple
            Capcaidad total de baterías, número de baterías en paralelo, número de baterías en serie

        """
        daily_ah = self.daily_ah() * self.amb_temp_multiplier
        total_battery_capacity = (daily_ah * self.days_auto) / self.dod
        n_bat_paralell = ceil(total_battery_capacity / self.amp_hour_rating)
        n_bat_series = ceil(self.nominal_voltage / self.batt_volt)
        return total_battery_capacity, n_bat_paralell, n_bat_series



from pv_sizing.dimension.pv import PVProduction

from pv_sizing.utils.load_example import example_irr, example_load
from pv_sizing.utils.pv_utils import init_inv

from pv_sizing.app_layout.applayout import app
import plotly.graph_objects as go
from dash import Dash, dcc, html, Input, Output, State
from plotly.subplots import make_subplots


@app.callback(
[Output(component_id='PV', component_property='figure')],
[Input(component_id='num_panels', component_property='value'),
Input(component_id='panel_price', component_property='value'),
Input(component_id='inverter_price', component_property='value'),
Input(component_id='addition_cost', component_property='value'),
Input(component_id='instalation_cost_perc', component_property='value'),
Input(component_id='select_tnoct', component_property='value'),
Input(component_id='panel_power', component_property='value'),
Input(component_id='gamma', component_property='value')])
def interactive_plot(num_panel = 2, panel_price = 260, inverter_price = 1300, additional_cost = 500, installation_cost_perc = 0.15,
                     tnoct = 42, panel_power = 450, gamma = -0.36):

    pv = PVProduction(irr_data=example_irr, load=example_load, tnoct=tnoct, gamma=gamma, panel_power=panel_power, num_panel=num_panel)

    initial_investment = init_inv(num_panel=num_panel, price_panel=panel_price,
                                    additional_cost=additional_cost, installation_cost_perc=installation_cost_perc,
                                    price_inverter=inverter_price)

    cashflow, van, tir = pv.economic_analysis(initial_investment)

    fig = make_subplots(rows=2, cols=1, horizontal_spacing = 1)

    fig.add_trace(
        go.Scatter(x = pv.myload_yearly.index, y = pv.myload_yearly.AE_kWh, name = 'Load'),
        row=1, col=1
    )

    fig.add_trace(go.Scatter(x = pv.myload_yearly.index, y = pv.myprod_yearly.kWh, name= 'PV production'),
        row=1, col=1
    )

    fig.add_trace(
        go.Bar(x=cashflow.index.values, y=cashflow['Accumulated cashflow'], name = 'Accumulated cashflow'),
        row=2, col=1
    )

    fig.update_xaxes(title_text="Time [h]", row=1, col=1)
    fig.update_yaxes(title_text="Energy [kWh]", row=1, col=1)

    fig.update_xaxes(title_text="Time [years]", row=2, col=1)
    fig.update_yaxes(title_text="Money [€]", row=2, col=1)

    fig.update_layout(legend=dict(
    orientation="h",
    yanchor="bottom",
    y=1.02,
    xanchor="right",
    x=1))

    fig.update_layout(
    paper_bgcolor="#f8f9fa"
    )

    return [fig]


from dash import Dash, dcc, html, Input, Output
import numpy as np

external_stylesheets = ['https://codepen.io/chriddyp/pen/bWLwgP.css']
app = Dash(__name__, external_stylesheets=external_stylesheets)

# the style arguments for the sidebar. We use position:fixed and a fixed width
SIDEBAR_STYLE = {
    "position": "fixed",
    "overflow": "scroll",
    "top": 0,
    "left": 0,
    "bottom": 0,
    "width": "13rem",
    "padding": "2rem 1rem",
    "background-color": "#f8f9fa",
}

# the styles for the main content position it to the right of the sidebar and
# add some padding.
CONTENT_STYLE = {
    "margin-left": "15rem",
    "margin-right": "0rem",
    "padding": "5rem 1rem",
    "background-color": "#f8f9fa",
}

sidebar = html.Div([
    html.H4("Select parameters", className="display-4"),
    html.Hr(),

    html.P('Number of panels',style={'display':'inline-block','margin-right':20}),
    dcc.Input(id="num_panels", type="number", placeholder="Numer of panels", value = 1, style={'width':'90%', 'marginRight':'10px', 'display':'inline-block'},
                 ),

    html.Hr(),

    html.P('Panel price',style={'display':'inline-block','margin-right':20}),
    dcc.Input(id="panel_price", type="number", placeholder="Panel price", value = 260 ,style={'width':'90%','marginRight':'10px', 'display':'inline-block'}
                 ),

    html.Hr(),
    
    html.P('Inverter price',style={'display':'inline-block','margin-right':20}),
    dcc.Input(id="inverter_price", type="number", placeholder="Inverter price", value = 1300 ,style={'width':'90%','marginRight':'10px', 'display':'inline-block'}
                 ),
        
    html.Hr(),

    html.P('Additional cost',style={'display':'inline-block','margin-right':20}),
    dcc.Input(id="addition_cost", type="number", placeholder="Additional price", value = 500 ,style={'width':'90%','marginRight':'10px', 'display':'inline-block'}
                 ),

    html.Hr(),

    html.P('Installation cost percantage',style={'display':'inline-block','margin-right':20}),
    dcc.Input(id="instalation_cost_perc", type="number", placeholder="Instalation percentage cost", value = 0.15 ,style={'width':'90%','marginRight':'10px', 'display':'inline-block'}
                 ),

    html.Hr(),

    html.P('Panel power',style={'display':'inline-block','margin-right':20}),
    dcc.Input(id="panel_power", type="number", placeholder="Instalation percentage cost", value = 450 ,style={'width':'90%','marginRight':'10px', 'display':'inline-block'}
                 ),

    html.Hr(),

    html.P('Gamma',style={'display':'inline-block','margin-right':20}),
    dcc.Input(id="gamma", type="number", placeholder="Instalation percentage cost", value = -0.36 ,style={'width':'90%','marginRight':'10px', 'display':'inline-block'}
                 ),

    html.Hr(),

    html.P('TNOC [ºC]',style={'display':'inline-block','margin-right':20}),
    dcc.Input(id="select_tnoct", type="number", placeholder="TNOCT [ºC]", value = 42 ,style={'width':'90%','marginRight':'10px', 'display':'inline-block'}
                 )],
                 
    style=SIDEBAR_STYLE,
)

# App layout
app.layout = html.Div([

    html.H1("Photovoltaic production", style={'text-align': 'center'}),

    html.Div([
    dcc.Graph(id='PV', figure={})
    ]),
    html.Br(), 
    sidebar
],
    style = CONTENT_STYLE)


import pandas as pd
import os

this_dir, this_filename = os.path.split(__file__)
DATA_PATH = os.path.join(this_dir, "example_data", "example_irr.csv")
example_irr = pd.read_csv(DATA_PATH, index_col='time')

this_dir, this_filename = os.path.split(__file__)
DATA_PATH = os.path.join(this_dir, "example_data", "example_load.csv")
example_load = pd.read_csv(DATA_PATH, index_col='time')





import matplotlib.pyplot as plt
import pandas as pd

import numpy as np
import numpy_financial as npf
from itertools import accumulate
from collections import deque

from pv_sizing.utils.pv_utils import performance_ratio, european_efficiency_inverter, index_tuple_to_datetime, oneyear_todatetimeindex, \
                                    idae_pv_prod, cell_temp
from pv_sizing.utils.irradiance import get_irradiance

from pv_sizing.utils.constants import fresnel_fixed

import warnings
from pandas.core.common import SettingWithCopyWarning




warnings.simplefilter(action="ignore", category=SettingWithCopyWarning)


class PVProduction:

    def __init__(self, load, irr_data,  tnoct, gamma, panel_power, num_panel, fresnel_eff = fresnel_fixed,
                 lat=None, lon=None, start_date=None, end_date=None, tilt=None,
                 surface_azimuth=None, freq='1H'):
        """
        Args:
            load: pd.DataFrame
                Datos horarios de carga obtenido de https://www.edistribucion.com/
            irr_data: pd.DataFrame
                Irradiancias (directa, difusa y reflejada) sobre el plano inclinado, temperatura
                media, velcodidad del viento horarios obtenido a partir de https://re.jrc.ec.europa.eu/pvg_tools/en/
            tnoct: int
                "Nominal operating cell temperature" facilitada por el fabricante
            gamma: float
                Coeficiente de pérdidas facilitado por el fabricante
            panel_power: int
                Potencia pico del panel fotovoltaico
            num_panel: int
                Número de paneles fotovoltaicos
            fresnel_eff: array_like
                Eficiencia fresnel por cada mes

        """

        if irr_data is None:
            self.irr_data = get_irradiance(lat, lon, start_date, end_date, tilt, surface_azimuth, freq)
            raise NotImplementedError('API is needed for hourly temperature')

        self.load = load
        self.irr_data = irr_data

        if len(self.load.columns) > 1:
            raise ValueError(f'DataFrame found with {len(self.load.columns)}, only 1 is needed.')

        if 'AE_kWh' not in self.load.columns.values:
            self.load.rename(columns={self.load.columns.values[0]: 'AE_kWh'}, inplace = True)

        if not isinstance(self.load.index, pd.DatetimeIndex) or not isinstance(self.load.index, pd.DatetimeIndex):
            try:
                self.load.index = pd.to_datetime(self.load.index)
                self.irr_data.index = pd.to_datetime(self.irr_data.index)
            except:
                raise TypeError('Index must be DateTimeIndex')
        
        self.fresnel_eff = fresnel_eff
        self.tnoct = tnoct
        self.gamma = gamma
        self.panel_power = panel_power
        self.num_panels = num_panel

        self.prod_data = self.pv_production()
        self.myload_yearly, self.myirr_yearly, self.myprod_yearly = self._yearly_load_and_irr_to_datetime_index()

    def mean_yearly_load_data(self):
        """
        Función para calcular la media anual de la carga.

        Returns: pd.DataFrame con la media anual de la carga
        """

        return oneyear_todatetimeindex(
            self.load.groupby([self.load.index.month, self.load.index.day, self.load.index.hour]).mean())

    def mean_yearly_irr_data(self):
        """
        Función para calcular la media anual de la irradiancia.

        Returns: pd.DataFrame con la media anual de la irradiancia
        """

        return oneyear_todatetimeindex(self.irr_data.groupby(
            [self.irr_data.index.month, self.irr_data.index.day, self.irr_data.index.hour]).mean())
    
    def mean_yearly_prod_data(self):
        """
        Función para calcular la media anual de la producción.

        Returns: pd.DataFrame con la media anual de la producción
        """

        return oneyear_todatetimeindex(self.prod_data.groupby(
            [self.prod_data.index.month, self.prod_data.index.day, self.prod_data.index.hour]).mean())
    
    def mean_hourly_load_data(self):
        """
        Función para calcular la media horaria de la carga.

        Returns: pd.DataFrame con la media horaria de la carga
        """
        return self.load.groupby([self.load.index.hour]).mean()

    def mean_hourly_irr_data(self):
        """
        Función para calcular la media horaria de la irradiancia.

        Returns: pd.DataFrame con la media horaria de la irradiancia
        """
        return self.irr_data.groupby([self.irr_data.index.hour]).mean()

    def mean_hourly_irr_data(self):
        """
        Función para calcular la media horaria de la producción.

        Returns: pd.DataFrame con la media horaria de la producción
        """
        return self.prod_data.groupby([self.prod_data.index.hour]).mean()

    def pv_production(self):
        """
        Función para crear un Data Frame de irradiancia el Performance Ratio de la instalción y la producción de
        energía para cada time step.

        Returns: pd.DataFrame con producción horaria
        """

        df_prod = pd.DataFrame(index = self.irr_data.index)

        self.irr_data['Irr'] = self.irr_data['Gb(i)'] + self.irr_data['Gd(i)'] + self.irr_data['Gr(i)']

        df_prod = cell_temp(df_prod, self.irr_data, self.tnoct)
        df_prod = performance_ratio(df_prod, self.gamma, df_prod.T_cell, self.fresnel_eff.mean())
        df_prod = idae_pv_prod(df_prod, self.irr_data, self.panel_power, self.num_panels)

        return df_prod

    def _yearly_load_and_irr_to_datetime_index(self):
        """
        Función para convertir el índice de la media anual de irradiancia y carga a formato datetime.

        Returns: tuple
            Tupla con Data Frames con la carga e irradiancia anual
        """
        myload = self.mean_yearly_load_data().set_index(
            pd.date_range('2019-01-01', '2020-01-01', freq='H', inclusive='left'))
        myirr = self.mean_yearly_irr_data().set_index(
            pd.date_range('2019-01-01', '2020-01-01', freq='H', inclusive='left'))
        myprod = self.mean_yearly_prod_data().set_index(
            pd.date_range('2019-01-01', '2020-01-01', freq='H', inclusive='left'))
        return myload, myirr, myprod

    def energy_balance(self):
        """
        Función para calcular el balance energético anual.

        Returns: tuple
            Tupla con el balance energético, energía comprada y energía vertida.
        """
        df_prod = self.pv_production()

        myload, myirr, myprod = self._yearly_load_and_irr_to_datetime_index()
        balance = myload.AE_kWh - myprod.kWh

        comprada = balance.loc[balance > 0].rename('from_grid')
        vertida = balance.loc[balance < 0].rename('into_grid')

        return balance, comprada, vertida

    def savings_from_pv(self, buy_price=0.32, sell_price=0.06):

        """
        Args:
            buy_price: float
                Precio de compra de energía.
            sell_price: float
                Precio de venta de energía.

        Returns: tuple
            Tupla con el coste de energía sin producción fotovoltaica, coste de energía con producción fotovoltacia,
            ahorro por compensación de energía fotovoltacia y ahorro final.
        """

        balance, comprada, vertida = self.energy_balance()
        load = self.mean_yearly_load_data().AE_kWh.to_frame()

        if not isinstance(buy_price, (int, float, str)):

            if isinstance(buy_price, pd.DataFrame) and not isinstance(buy_price.index, pd.DatetimeIndex):
                try:
                    buy_price.index = pd.to_datetime(buy_price.index)
                except:
                    raise TypeError(f"Found type {type(buy_price.index)} but pd.DatetimeIndex was expected.")

            if len(buy_price) != 24:
                raise ValueError(f"{type(buy_price)} found with length {len(buy_price)} but 24 was expected.")

            while buy_price.index[0].hour != self.load.index[0].hour: #Rotamos la lista hasta que la primera hora del precio coincida con la primera hora de la carga.
                dq = deque(buy_price.index)
                dq.rotate(1)
                buy_price.index = dq

            lst_buy_price = buy_price.loc[:, buy_price.columns[0]].tolist()  #Pasamos a lista los valores del precio
            lst_buy_price *= len(load)//len(lst_buy_price) #Hacemos que la lista del precio coincida en longitud con el dataframe
            load['buy_price'] = lst_buy_price #Creamos la columna en el dataframe con el precio horario.

            total_df = pd.concat([load, comprada, vertida], axis=1).fillna(0) #Concatenamos la carga, energía comprada y vertida en un mismo DataFrame

            coste_energia_actual = sum(total_df.buy_price * total_df.AE_kWh)
            coste_energia_pv = sum(total_df.buy_price * total_df.from_grid)
            compensacion_pv = -sum(total_df.buy_price * total_df.into_grid)
        else:
            coste_energia_actual = sum(buy_price * load.AE_kWh)
            coste_energia_pv = sum(buy_price * comprada)
            compensacion_pv = -sum(sell_price * vertida)

        ahorro = coste_energia_actual - coste_energia_pv + compensacion_pv

        return coste_energia_actual, coste_energia_pv, compensacion_pv, ahorro

    def economic_analysis(self, init_inversion, buy_price=0.32, sell_price=0.06,ibi=None, oym_perc=0.02, proj_duration=25, ipc=0.04,
                          discount_rate=0.02):
        """
        Args:
            init_inversion: float
                Inversión inicial para la instalación.
            ibi: float
                Descuento dependiente del municipio en el que se haga la instalación.
            oym_perc: float
                Porcentaje de la inversión inicial equivalente a gastos en operación y mantenimiento.
            proj_duration: int
                Duración del proyecto.
            ipc: float
                Porecentaje correspondiente a la inflación y devaluación del dinero.
            discount_rate: float
                Coste de capital que se aplica para determinal el valor presente d eun pago futuro.


        Returns: tuple
            El primer valor corresponde a Data Frame con el cashflow, segundo valor corresponde a VAN y el último a TIR.
        """
        years = np.arange(1, proj_duration + 1)
        ahorro = self.savings_from_pv(buy_price=buy_price, sell_price=sell_price)[-1]
        oym = init_inversion * oym_perc

        initial_inversion = np.zeros(len(years))
        initial_inversion[0] = init_inversion

        duration_project_savings = np.array([ahorro * ((1 + ipc) ** (year - 1)) for year in years])
        duration_project_oym = np.array([oym * ((1 + oym_perc) ** (year - 1)) for year in years])

        cf = - initial_inversion - duration_project_oym + duration_project_savings
        a_cf = np.array(list(accumulate(cf)))

        if ibi:
            raise ValueError('This option haven´t been added yet.')

        df_cf = pd.DataFrame(
            {'Initial inversion': initial_inversion, 'Operation and maintanence': duration_project_oym, 'Savings': duration_project_savings, 'Cashflow': cf,
             'Accumulated cashflow': a_cf})

        return df_cf, npf.npv(discount_rate, cf), npf.irr(cf)
        
    def plot(self, cashflow):
        """
        Args:
            cashflow: pd.DataFrame
                Data Frame con el cashflow acumulado del proyecto.
        """

        myload, myirr, myprod = self._yearly_load_and_irr_to_datetime_index()

        fig, ax = plt.subplots(2)
        cashflow.plot.bar(ax=ax[1])
        myload.AE_kWh.to_frame().rename(columns={'AE_kWh':'Load'}).plot(ax=ax[0])
        myprod.kWh.to_frame().rename(columns={'kWh':'Production'}).plot(ax=ax[0])

        ax[0].set(title = "Yearly photovoltaic production and load", ylabel="Energy [kWh]", xlabel="Time [hours]")
        ax[1].set(title = "Cashflow", xlabel="Time [years]", ylabel= "Money [€]")

        ax[0].legend()

        plt.tight_layout()
        plt.show()



def remove_leap_day(df):
    """

    Args:
        df:

    Returns:

    """
    return df[~((df.index.month == 2) & (df.index.day == 29))]

def remove_last_row(df):
    """

    Args:
        df:

    Returns:

    """
    return df[df.Fecha != 0]

def remove25hformat(load):
    """

    Args:
        load:

    Returns:

    """

    load = load.loc[load.Hora <= 24]
    load.loc[load.Hora == 24, 'Hora'] = 0
    load = remove_last_row(load)

    load['Hora'] = load['Hora'].apply(
        lambda x: '0' + str(x) if len(str(x)) == 1 else x).astype(str)

    load['time'] = load['Fecha'].astype(str) + '-' + load['Hora']
    load['time'] = pd.to_datetime(load['time'], format='%d/%m/%Y-%H')
    return load

def from24to00(load):
    """
    Función para pasar de 24:00 a 00:00 y así coincidior con 

    Args:
        load:

    Returns:

    """
    load['Hora'] = load['Hora'].apply(
        lambda x: '0' + str(x) if len(str(x)) == 1 else x).astype(str)
    load['time'] = load['Fecha'].astype(str) + '-' + load['Hora']
    load['time'] = pd.to_datetime(load['time'], format='%d/%m/%Y-%H')
    return load


from setuptools import setup, find_packages, setuptools

setup(
    long_description=open("README.md").read(),
    long_description_content_type='text/markdown',
    name='pv_sizing',
    version='1.1.2',
    license='MIT',
    author="Kiril Ivanov Kurtev",
    author_email='brakisto2015@gmail.com',
    packages=['pv_sizing/dimension', 'pv_sizing/utils', 'pv_sizing/web_scrapping', 'pv_sizing/app_layout'],
    package_dir={'': 'src'},
    url='https://github.com/brakisto/PV-sizing',
    keywords='photovoltaic sizing batteries financial analysis',
    install_requires=[
          'numpy',
          'numpy-financial',
          'pandas',
          'pvlib',
          'selenium',
          'plotly',
          'dash'
      ],
    include_package_data=True,
    package_data={'pv_sizing/utils': ['example_data/*.csv']},
      )





import pandas as pd

def cell_temp(df_prod, irr_data, tnoct):
    """_summary_

    Args:
        df_prod (_type_): _description_
        irr_data (_type_): _description_
        tnoct (_type_): _description_

    Returns:
        _type_: _description_
    """
    # TEMPERATURA CÉLULA
    df_prod['T_cell'] = irr_data['T2m'] + irr_data['Irr'] * (tnoct - 20) / 800
    return df_prod

def idae_pv_prod(df_prod, df_irr, panel_power, num_panels):

    """_summary_

    Args:
        df_prod (_type_): _description_
        df_irr (_type_): _description_
        panel_power (_type_): _description_
        num_panels (_type_): _description_

    Returns:
        _type_: _description_
    """
    # PRODUCCION
    df_prod['Wh'] = df_prod['PR'] * df_irr['Irr'] * panel_power * num_panels / 1000 # E = GR [kW/m2] * PR * P [W] * num_panels / 1 kW/m2 /// Como son intervalos de 1 hora obtenemos Wh 
    df_prod['kWh'] = df_prod.Wh / 1e3
    df_prod['MWh'] = df_prod.Wh / 1e6
    return df_prod

def performance_ratio(df_prod_hourly, gamma, t_cell_hourly, mean_fresnel_eff):
    """_summary_

    Args:
        df_prod_hourly (pd.DataFrame): _description_
        gamma (float): _description_
        t_cell_hourly (numpy.array or pd.Series or pd.DataFrame): _description_
        mean_fresnel_eff (float): _description_

    Returns:
        pd.DataFrame: _description_
    """
    
    # PERFORMANCE RATIO
    df_prod_hourly['PRtemp'] = (1 + gamma * (t_cell_hourly - 25) / 100)

    # OBTENIDO A PARTIR DE LA FUNCIÓN PARA LA EFICIENCIA EUROPEA
    df_prod_hourly['PRfres'] = mean_fresnel_eff
    # CAIDA DE TENSIÓN DE 0,8% PARA CC
    df_prod_hourly['PRCC'] = 0.992
    df_prod_hourly['PRdisp'] = 1
    df_prod_hourly['PRCC/CA'] = european_efficiency_inverter(eta5=60, eta10=80, eta20=89, eta30=91, eta50=92,
                                                            eta100=93) / 100
    # CAIDA DEL 1,5% PARTE AC
    df_prod_hourly['PRAC'] = 0.985
    df_prod_hourly['PR'] = df_prod_hourly['PRtemp'] * df_prod_hourly['PRfres'] * df_prod_hourly['PRdisp'] * \
                            df_prod_hourly['PRCC'] * df_prod_hourly['PRCC/CA'] * df_prod_hourly['PRAC']
    
    return df_prod_hourly


def european_efficiency_inverter(eta5, eta10, eta20, eta30, eta50, eta100):
    """
    Función para el cáculo de la Eficiencia Europea de un inversor.
    La Eficiencia Europea es una eficiencia de operación promedio
    sobre una distribución de energía anual correspondiente al clima
    de Europa Media y ahora se hace referencia en casi cualquier hoja de datos del inversor.

    El valor de esta eficiencia ponderada se obtiene asignando un porcentaje de tiempo a
    los residuos del inversor en un rango de funcionamiento dado.

    Suele venir en la hoja técnica del inversor.
    """
    return 0.03 * eta5 + 0.06 * eta10 + 0.13 * eta20 + 0.1 * eta30 + 0.48 * eta50 + 0.2 * eta100

def index_tuple_to_datetime(df):
    a = pd.DataFrame(df.index.values.tolist(), columns=['month', 'day', 'hour'])
    df['time'] = pd.to_datetime(a, format='%m%d%H')
    return df

def init_inv(num_panel, price_panel, additional_cost, installation_cost_perc, price_inverter):
    inversion_inicial = num_panel*price_panel + price_inverter + additional_cost
    return inversion_inicial + inversion_inicial*installation_cost_perc

def oneyear_todatetimeindex(df):
    if not isinstance(df, pd.DataFrame):
        df = df.to_frame()
    return df.set_index(pd.date_range('2019-01-01', '2020-01-01', freq='H', inclusive='left'))
